
capture program drop sftreat_GMM4_jup
program define sftreat_GMM4_jup

/*
For the *_jup version, comment out the warning message. The *_jup version is applied to
the demo dataset and demo model which does not have convergence problem.

if c(version) >= 14 {
   di in red "The program -sftreat_GMM4- does not run correctly in Stata version 14 and above."
   di in red "Please use Stata versions 12.x or 13.x."
   exit 111
}
*/

  version 12
  syntax varlist if, at(name) ///
         mylhs(varlist) myxvar(varlist) myzvar(varlist) mywght(varlist) mywghtB(varlist) myp(varlist) mypB(varlist) [GNOCONS]


*****************************************************
**** construct functions for the GMM estimation *****
*****************************************************

tempvar myxb

quie gen double `myxb' = 0 `if'

local i = 1
foreach X in `myxvar' {
  quie replace `myxb' = `myxb' + `at'[1,`i']*`X'  `if'
  local i = `i' + 1
}
quie replace `myxb' = `myxb' + `at'[1,`i']  `if'  /* the constant */

 *** now repeat the above block again to generate a copy for b_10

tempvar myxbB

quie gen double `myxbB' = 0 `if'

local i = `i' + 1         /* the i continues from the above */
foreach X in `myxvar' {
  quie replace `myxbB' = `myxbB' + `at'[1,`i']*`X'  `if'
  local i = `i' + 1
}
quie replace `myxbB' = `myxbB' + `at'[1,`i']  `if'  /* the constant */
local nofx = `i'


 *************************************************
    **** do the same for the g function ****

tempvar myzd
quie gen double `myzd' = 0 `if'

local i = `i' + 1
foreach X in `myzvar' { /* the i here continues from the above */
  quie replace `myzd' = `myzd' + `at'[1,`i']*`X' `if'
  local i = `i' + 1
}

if "`gnocons'" == "" {  /* need constant */
  quie replace `myzd' = `myzd' + `at'[1,`i'] `if'
}
else {
  local i = `i' -1
}


   **** another copy for gamma_10

tempvar myzdB
quie gen double `myzdB' = 0 `if'

local i = `i' + 1
foreach X in `myzvar' { /* the i here continues from the above */
  quie replace `myzdB' = `myzdB' + `at'[1,`i']*`X' `if'
  local i = `i' + 1
}

if "`gnocons'" == "" {  /* need constant */
   quie replace `myzdB' = `myzdB' + `at'[1,`i'] `if'
}
else {
   local i = `i' - 1
}


local nofz = `i' - `nofx'


******************************
**** construct the functions
******************************

tempvar gfunpart1 gfunpart2

 quie gen double `gfunpart1' = sqrt(2/_pi)*exp(`myzd') `if'
 quie gen double `gfunpart2' = sqrt(2/_pi)*exp(`myzdB') `if'

tempvar hfun hfunB gfun gfunB omega omegaB

 quie gen double `hfun'  = `myp'*(`myxb')  + (1-`myp')*(`myxbB') `if'
 quie gen double `hfunB' = `mypB'*(`myxb') + (1-`mypB')*(`myxbB') `if'

 quie gen double `gfun'  =  `myp'*(`gfunpart1')  + (1-`myp')*(`gfunpart2') `if'
 quie gen double `gfunB' = `mypB'*(`gfunpart1') + (1-`mypB')*(`gfunpart2') `if'

 quie gen double `omega' = `mylhs' - (`hfun') + (`gfun') `if'
 quie gen double `omegaB'= `mylhs' - (`hfunB') + (`gfunB') `if'


  *** the moments of beta *****

 local nofxz = `nofx' + `nofz'  /* total parameters, i.e., beta_11, beta_10, gamma_11, gamma_10 */

 forvalues i = 1/`nofxz' {
  local eq`i': word `i' of `varlist'  /* note this varlist is determined by nequation() and is not really from the dataset, so nothing get replaced */
 }

    /* this part for beta_11 */

 local i = 1
 foreach X in `myxvar' {
   quie replace `eq`i'' = `mywght'*(`omega')*(-`myp'*`X') + `mywghtB'*(`omegaB')*(-`mypB'*`X') `if'
   local i = `i' + 1
 }
  quie replace `eq`i'' = `mywght'*(`omega')*(-`myp') + `mywghtB'*(`omegaB')*(-`mypB') `if'  /* the constant */
  local i = `i' + 1

    /* this part for beta_10 */

 foreach X in `myxvar' {
   quie replace `eq`i'' = `mywght'*(`omega')*(-(1-`myp')*`X') + `mywghtB'*(`omegaB')*(-(1-`mypB')*`X') `if'
   local i = `i' + 1
 }
  quie replace `eq`i'' = `mywght'*(`omega')*(-(1-`myp')) + `mywghtB'*(`omegaB')*(-(1-`mypB')) `if'  /* the constant */
  local i = `i' + 1

 *** the moment of delta (or called gamma) ******

   /* this part for delta_11 */

 foreach X in `myzvar' {  /* here the i follows from the above */
   quie replace `eq`i'' = `mywght'*(`omega')*((`myp')*(`gfunpart1'))*(`X') + `mywghtB'*(`omegaB')*((`mypB')*(`gfunpart1'))*(`X') `if'
   local i = `i' + 1
 }

 if "`gnocons'" == "" {  /* need constant */
   quie replace `eq`i'' = `mywght'*(`omega')*(`myp'*`gfunpart1') + `mywghtB'*(`omegaB')*(`mypB'*`gfunpart1') `if'
   local i = `i' + 1
 }

   /* this part for delta_10 */

 foreach X in `myzvar' {  /* here the i follows from the above */
   quie replace `eq`i'' = `mywght'*(`omega')*((1-`myp')*`gfunpart2')*(`X') + `mywghtB'*(`omegaB')*((1-`mypB')*`gfunpart2')*(`X') `if'
   local i = `i' + 1
 }

 if "`gnocons'" == "" {  /* need constant */
   quie replace `eq`i'' = `mywght'*(`omega')*((1-`myp')*`gfunpart2') + `mywghtB'*(`omegaB')*((1-`mypB')*`gfunpart2') `if'
}

end
